@@ -3,7 +3,7 @@
 import argparse
 import os
 import sys
-
+import platform
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
@@ -11,15 +11,39 @@ import matplotlib
 
 matplotlib.rcParams['agg.path.chunksize'] = 100000
 
+def find_header(fname):
+    """
+    Find the header of any length
+    Args:
+        fname:
+
+    Returns:
+    """
+    with open(fname) as fp:
+        lines = fp.readlines()
+        result = None
+
+        for i, line in enumerate(lines):
+            if ',' in line:
+                result = i
+                break
+
+        return result
+
 
 def plot_hi_res(fname=None, df=None):
     """
     Plots the timeseries, the depth corrected, and the depth data
     """
+    if 'Linux' in platform.platform():
+        matplotlib.use('TkAgg')
+        
     names = {'Sensor1': 'Hardness', 'Sensor2': 'Ambient NIR', 'Sensor3': 'Active NIR'}
 
     if fname is not None:
-        df = pd.read_csv(fname, header=5)
+        header = find_header(fname)
+        print(header)
+        df = pd.read_csv(fname, header=header)
 
     f, axes = plt.subplots(1, 3)
     # fig = matplotlib.pyplot.gcf()
