@@ -9,7 +9,7 @@ from colorama import init
 from termcolor import colored
 
 
-class Messages():
+class Messages:
     init()
 
     def msg(self, str_msg, context_str=None):
@@ -147,7 +147,7 @@ def print_helpme(help_str, help_dict):
             for k, v in help_dict.items():
                 if v is None:
                     no_help += 1
-                    v = ('No help documentation.')
+                    v = 'No help documentation.'
 
                 out_str += '\n{0:<25} {1:<25}\n'.format(k, v)
         if no_help == len(help_dict.keys()):
@@ -156,7 +156,9 @@ def print_helpme(help_str, help_dict):
         no_doc = True
 
     if no_doc:
-        out_str = ('\nNo help documentation.\nPlease email micah@adventuredata.com or file an issue at https://github.com/Adventuredata/radicl/issues\n')
+        out_str = (
+            '\nNo help documentation.\nPlease email info@adventuredata.com or file an issue at '
+            'https://github.com/Adventuredata/radicl/issues\n')
         print(help_dict)
 
     # Doctor up the print out
@@ -166,15 +168,25 @@ def print_helpme(help_str, help_dict):
     print(print_able)
 
 
-def get_logger(name, level='DEBUG', ext_logger=None,):
+def get_logger(name, debug=False, ext_logger=None, ):
     """
+    Args:
+        name: Name of the logger.
+        debug: Bool whether to show debug statements
+        ext_logger: Use ext logger to pass an already instantiated logger
 
+    Returns:
+        log: Instantiated logger
     """
     fmt = fmt = '%(name)s %(levelname)s %(message)s'
     if ext_logger is None:
         log = logging.getLogger(name)
     else:
         log = ext_logger
+    if debug:
+        level = 'DEBUG'
+    else:
+        level = 'INFO'
 
     coloredlogs.install(fmt=fmt, level=level, logger=log)
     return log
